package org.oscy.tapestry.breadcrumbs.internal.services.impl;

import java.lang.annotation.Annotation;
import java.util.Collection;
import java.util.Iterator;

import org.apache.tapestry5.runtime.Component;
import org.apache.tapestry5.services.ApplicationStateManager;
import org.apache.tapestry5.services.ComponentSource;
import org.apache.tapestry5.services.PageRenderRequestParameters;
import org.oscy.tapestry.breadcrumbs.annotation.BreadCrumb;
import org.oscy.tapestry.breadcrumbs.internal.services.BreadCrumbsService;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbAttribute;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbElement;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbRule;
import org.oscy.tapestry.breadcrumbs.session.BreadCrumbsList;
import org.oscy.tapestry.breadcrumbs.session.impl.BreadCrumbsListImpl;
import org.oscy.tapestry.breadcrumbs.utils.AntPathMatcher;


/**
 * 
 * Implementation class for {@link BreadCrumbsService}
 *
 */
public class BreadCrumbsServiceImpl implements BreadCrumbsService
{

	private final ApplicationStateManager applicationStateManager;
	private final AntPathMatcher patternMatcher;
	private final Collection<BreadCrumbRule> pathRules;
	private final ComponentSource componentSource;
	
	/**
	 * Default constructor for that service.All the parameters are injected through 
	 * tapestry-ioc.
	 * 
	 * @param applicationStateManager 
	 * @param pathRules
	 * @param componentSource
	 */

	public BreadCrumbsServiceImpl(ApplicationStateManager applicationStateManager, Collection<BreadCrumbRule> pathRules, ComponentSource componentSource) {
		this.applicationStateManager = applicationStateManager;
		this.patternMatcher = createPathMatcher();
		this.pathRules = pathRules;
		this.componentSource = componentSource;
	}

	public void addToBreadCrumbs(PageRenderRequestParameters params) {

		if (params.isLoopback()) {
			throw new RuntimeException("IS loopback");
		}
		BreadCrumbsListImpl breadCrumbsList = (BreadCrumbsListImpl) applicationStateManager.get(BreadCrumbsList.class);
		String pageName = params.getLogicalPageName();
		boolean ignorePage = false;
		boolean resetCrumbs = false;
		Iterator<BreadCrumbRule> iter = pathRules.iterator();
		while (iter.hasNext()) {
			BreadCrumbRule filter = iter.next();
			if (patternMatcher.matches(filter.getPattern(), pageName)) {
				BreadCrumbAttribute attribute = filter.getAttribute();
				if (attribute == BreadCrumbAttribute.IGNORE) {
					ignorePage = true;
				}

				if (attribute == BreadCrumbAttribute.RESET) {
					resetCrumbs = true;
				}
			}
		}
		Component page = componentSource.getPage(pageName);
		BreadCrumb breadCrumb = findAnnotation(page.getClass(), BreadCrumb.class);
		if (breadCrumb != null) {
			if (breadCrumb.ignore()) {
				ignorePage = true;
			}

			if (breadCrumb.reset()) {
				resetCrumbs = true;
			}
		}

		if (resetCrumbs) {
			breadCrumbsList.clear();
		}

		if (ignorePage) {
			return;
		}
		breadCrumbsList.add(new BreadCrumbElement(pageName, params.getActivationContext()));
	}

	private AntPathMatcher createPathMatcher() {
		return new AntPathMatcher()
		{
			public boolean matches(String pattern, String source) {
				return super.matches(pattern, source.toLowerCase());
			}
		};
	}
	
	/*
	 * Loop through the class and superclass and search for a specific annotation
	 */
	private <T extends Annotation> T findAnnotation(final Class<?> clazz, final Class<T> annotation) {

		Class<?> current = clazz;
		while (current != Object.class) {
			T result = clazz.getAnnotation(annotation);
			if(result != null) {
				return result;
			}
			current = current.getSuperclass();
		}
		return null;
	}
}
